use actix_web::{get, post, web, App, HttpResponse, HttpServer, Responder};
use ledger_utxo::{AccountId, AssetDefinition, AssetManager, Status, TransactionId};
use serde::{Deserialize, Serialize};
use serde_json::json;
use sqlx::sqlite::SqlitePoolOptions;

#[derive(Deserialize)]
pub struct Movement {
    pub account: AccountId,
    pub amount: String,
    pub asset: String,
}

#[derive(Deserialize)]
pub struct Transaction {
    pub debit: Vec<Movement>,
    pub credit: Vec<Movement>,
    pub memo: String,
    pub status: Status,
}

#[derive(Serialize, Deserialize)]
struct Item {
    id: i32,
    name: String,
}

#[get("/tx/{id}")]
async fn get_item(info: web::Path<TransactionId>, ledger: web::Data<Ledger>) -> impl Responder {
    // Retrieve an item by ID from a database or another data source.
    // For this example, we'll return a hardcoded item.

    return HttpResponse::Ok().json(json!(info.0));
    let tx = ledger._inner.get_transaction(&info.0).await;
    println!("{:?}", tx);

    if let Ok(tx) = tx {
        HttpResponse::Ok().json(tx)
    } else {
        HttpResponse::BadRequest().json(json!({"error": "not found"}))
    }
}

#[post("/items")]
async fn create_item(item: web::Json<Transaction>, _ledger: web::Data<Ledger>) -> impl Responder {
    // Insert the item into a database or another data source.
    // For this example, we'll just echo the received item.
    HttpResponse::Created().json(json!({"test": "true"}))
}

pub struct Ledger {
    _inner: ledger_utxo::Ledger<'static, ledger_utxo::Batch<'static>, ledger_utxo::Sqlite<'static>>,
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    let asset_manager = AssetManager::new(vec![
        AssetDefinition::new(1, "BTC", 8),
        AssetDefinition::new(2, "USD", 4),
    ]);

    let rt = tokio::runtime::Builder::new_multi_thread()
        .worker_threads(10)
        .enable_all()
        .build()
        .expect("Failed to create Tokio runtime");

    rt.block_on(async {
        let pool = SqlitePoolOptions::new()
            .max_connections(1)
            .idle_timeout(None)
            .max_lifetime(None)
            .connect(":memory:")
            .await
            .expect("pool");

        HttpServer::new(move || {
            let storage = ledger_utxo::Sqlite::new(pool.clone(), asset_manager.clone());
            let ledger = ledger_utxo::Ledger::new(storage);
            App::new()
                .data(Ledger { _inner: ledger })
                .service(get_item)
                .service(create_item)
        })
        .bind("127.0.0.1:8080")?
        .run()
        .await
    })
}
